import argparse
import tabulate
import json
import logging
import scanResultUtil
import sys
from operator import itemgetter
import re

# This CLI should probably be merger with the cbctl one, but this is currently
# not optimal since that one has not only analyser logic, but also result
# downloading logic

FIPS_CHECK_MODE = "info"  # "info" or "fail"

parser = argparse.ArgumentParser(
    description='Container image analyser for twistlock')
parser.add_argument(
    '-t',
    '--target_scan',
    help='JSON scan output of the target image',
    required=True, type=str
)
parser.add_argument(
    '-b',
    '--base_scan',
    help='JSON scan output of the base image',
    required=False, type=str
)
parser.add_argument(
    '-jt',
    '--json_target',
    help='json file of the image being scanned',
    required=False, type=str
)
parser.add_argument(
    '-jb',
    '--json_base',
    help='json file of the baseline image',
    required=False, type=str
)
parser.add_argument(
    '-oh',
    '--output_html',
    help='Output results into an HTML file',
    required=False,
    type=str
)
parser.add_argument(
    '-oj',
    '--output_json',
    help='Output results into a JSON file',
    required=False, type=str
)
parser.add_argument(
    '-fo',
    '--fips_openssl',
    help='Status from FIPS check- OpenSSL',
    required=False,
    choices=[0, 1, 2, 3, 4],
    default=0,
    type=int
)
parser.add_argument(
    '-fj',
    '--fips_java',
    help='Status from FIPS check- Java',
    required=False,
    choices=[0, 1, 2, 3, 4],
    default=0,
    type=int
)
parser.add_argument(
    '-fp',
    '--fips_python',
    help='Status from FIPS check- Python',
    required=False,
    choices=[0, 1, 2, 3, 4],
    default=0,
    type=int
)
parser.add_argument(
    '-fg',
    '--fips_golang',
    help='Status from FIPS check- Golang',
    required=False,
    choices=[0, 1, 2, 3, 4],
    default=0,
    type=int
)
parser.add_argument(
    '-fn',
    '--fips_nodejs',
    help='Status from FIPS check- NodeJS',
    required=False,
    choices=[0, 1, 2, 3, 4],
    default=0,
    type=int
)
parser.add_argument(
    "-io",
    "--info_os",
    help="Container OS info from /etc/os-release",
    required=False,
    type=str,
)

args = vars(parser.parse_args())

JSON_TARGET_FILE = None
if args['json_target']:
    JSON_TARGET_FILE = args['json_target']

JSON_BASE_FILE = None
if args['json_base']:
    JSON_BASE_FILE = args['json_base']

HTML_FILE = None
if args["output_html"]:
    HTML_FILE = args["output_html"]

JSON_FILE = None
if args["output_json"]:
    JSON_FILE = args["output_json"]

logging.basicConfig(filename="scan.log",
                    format="%(asctime)s,%(msecs)d %(levelname)-8s "
                    "[%(filename)s:%(lineno)d] %(message)s",
                    datefmt="%Y-%m-%d:%H:%M:%S",
                    level=logging.DEBUG)


logging.debug(f"Loading JSON file: {args['target_scan']}")
try:
    f_json = open(args["target_scan"])
    twistlock_json = json.load(f_json)
    compliances = twistlock_json['results'][0]['compliances']
    f_json.close()
except Exception as e:
    compliances = []
    logging.error(f"Error loading JSON file: {e}")

try:
    f_baseline_json = open(args["base_scan"])
    baseline_json = json.load(f_baseline_json)
    f_baseline_json.close()
except Exception as e:
    baseline_json = {}

# Check FIPS
fips = {}
for c in ["openssl", "java", "python", "golang", "nodejs"]:
    scanResultUtil.fips_checks[c] = args[f"fips_{c}"]

    # Remove shell colours
    fips[c] = re.sub('\x1b[^m]*m', '', scanResultUtil.FIPS_STATUS[scanResultUtil.fips_checks[c]])

    if scanResultUtil.fips_checks[c] == 2:
        scanResultUtil.scan_status[FIPS_CHECK_MODE] = True
        scanResultUtil.status_message[FIPS_CHECK_MODE][4] = "FIPS check failed\n"


parsed_baseline_report = scanResultUtil.parse_twistlock(baseline_json)
parsed_report = scanResultUtil.parse_twistlock(twistlock_json)

diffed_report = scanResultUtil.report_diff(parsed_report, parsed_baseline_report)

# Let's normalize the reports
diffed_report = scanResultUtil.normalize_report(diffed_report)
parsed_baseline_report = scanResultUtil.normalize_report(parsed_baseline_report)

# Only fail/warn on the vulnerabilities in the target image
scanResultUtil.analyze_vulns(diffed_report)

# Check if scanned images is based on any of the approved images
scanResultUtil.check_image_base(JSON_TARGET_FILE, JSON_BASE_FILE)

# Show the os info table
scanResultUtil.show_info_os_table(HTML_FILE, args.get("info_os", ""))

scanResultUtil.show_fips_table(HTML_FILE)

# Save the normalized JSON report
normalized_output = scanResultUtil.normalize_output(diffed_report)
normalized_output["metadata"]["fips"] = fips
scanResultUtil.save_json_output(normalized_output, JSON_FILE)

# Check for malware and miners
scanResultUtil.check_compliance_twistcli(compliances, notify=True)
scanResultUtil.show_malware_table(HTML_FILE)

scanResultUtil.show_vuln_table(diffed_report, HTML_FILE)

# ToDo: we might want to save these results
if not scanResultUtil.show_report(diffed_report, parsed_baseline_report, HTML_FILE):
    sys.exit(1)
